@@ -12,6 +12,7 @@ import logging
 
 log = logging.getLogger(__name__)
 
+
 def has_identity(xmpp, jid, identity, on_true=None, on_false=None):
     def _cb(iq):
         ident = lambda x: x[0]
@@ -20,29 +21,16 @@ def has_identity(xmpp, jid, identity, on_true=None, on_false=None):
             on_true()
         if not res and on_false is not None:
             on_false()
-    xmpp.plugin['xep_0030'].get_info(jid=jid, callback=_cb)
 
-def get_version(xmpp, jid, callback=None, **kwargs):
-    def handle_result(res):
-        if res and res['type'] != 'error':
-            ret = res['software_version'].values
-        else:
-            ret = False
-        if callback:
-            callback(ret)
-        return ret
-    iq = xmpp.make_iq_get(ito=jid)
-    iq['query'] = 'jabber:iq:version'
-    result = iq.send(callback=handle_result if callback else None)
-    if not callback:
-        return handle_result(result)
+    xmpp.plugin['xep_0030'].get_info(jid=jid, callback=_cb)
 
 
 def get_room_form(xmpp, room, callback):
     def _cb(result):
         if result["type"] == "error":
             return callback(None)
-        xform = result.xml.find('{http://jabber.org/protocol/muc#owner}query/{jabber:x:data}x')
+        xform = result.xml.find(
+            '{http://jabber.org/protocol/muc#owner}query/{jabber:x:data}x')
         if xform is None:
             return callback(None)
         form = xmpp.plugin['xep_0004'].build_form(xform)
@@ -53,6 +41,7 @@ def get_room_form(xmpp, room, callback):
     iq.append(query)
     iq.send(callback=_cb)
 
+
 def _filter_add_receipt_request(self, stanza):
     """
     Auto add receipt requests to outgoing messages, if:
@@ -79,7 +68,7 @@ def _filter_add_receipt_request(self, stanza):
     if stanza['request_receipt']:
         return stanza
 
-    if not stanza['type'] in self.ack_types:
+    if stanza['type'] not in self.ack_types:
         return stanza
 
     if stanza['receipt']:
@@ -94,4 +83,3 @@ def _filter_add_receipt_request(self, stanza):
 
     stanza['request_receipt'] = True
     return stanza
-
