@@ -3,7 +3,6 @@ Define the PluginConfig and Plugin classes, plus the SafetyMetaclass.
 These are used in the plugin system added in poezio 0.7.5
 (see plugin_manager.py)
 """
-import os
 from functools import partial
 from configparser import RawConfigParser
 from poezio.timed_events import TimedEvent, DelayedEvent
@@ -13,12 +12,14 @@ import traceback
 import logging
 log = logging.getLogger(__name__)
 
+
 class PluginConfig(config.Config):
     """
     Plugin configuration object.
     They are accessible inside the plugin with self.config
     and behave like the core Config object.
     """
+
     def __init__(self, filename, module_name, default=None):
         config.Config.__init__(self, filename, default=default)
         self.module_name = module_name
@@ -41,7 +42,7 @@ class PluginConfig(config.Config):
 
     def read(self):
         """Read the config file"""
-        RawConfigParser.read(self, self.file_name)
+        RawConfigParser.read(self, str(self.file_name))
         if not self.has_section(self.module_name):
             self.add_section(self.module_name)
 
@@ -59,9 +60,8 @@ class PluginConfig(config.Config):
     def write(self):
         """Write the config to the disk"""
         try:
-            fp = open(self.file_name, 'w')
-            RawConfigParser.write(self, fp)
-            fp.close()
+            with self.file_name.open('w') as fp:
+                RawConfigParser.write(self, fp)
             return True
         except IOError:
             return False
@@ -81,8 +81,10 @@ class SafetyMetaclass(type):
                     raise
                 elif SafetyMetaclass.core:
                     log.error('Error in a plugin', exc_info=True)
-                    SafetyMetaclass.core.information(traceback.format_exc(), 'Error')
+                    SafetyMetaclass.core.information(traceback.format_exc(),
+                                                     'Error')
                     return None
+
         return helper
 
     def __new__(meta, name, bases, class_dict):
@@ -92,10 +94,12 @@ class SafetyMetaclass(type):
                     class_dict[k] = SafetyMetaclass.safe_func(v)
         return type.__new__(meta, name, bases, class_dict)
 
-class PluginWrap(object):
+
+class PluginWrap:
     """
     A wrapper to implicitly pass the module name to PluginAPI
     """
+
     def __init__(self, api, module):
         self.api = api
         self.module = module
@@ -105,7 +109,8 @@ class PluginWrap(object):
         module = object.__getattribute__(self, 'module')
         return partial(getattr(api, name), module)
 
-class PluginAPI(object):
+
+class PluginAPI:
     """
     The public API exposed to the plugins.
     Its goal is to limit the use of the raw Core object
@@ -160,7 +165,7 @@ class PluginAPI(object):
 
         :param datetime.datetime date: The time at which the handler must be executed
         :param function callback: The handler that will be executed
-        :param \*args: Optional arguments passed to the handler.
+        :param args: Optional arguments passed to the handler.
         :return: The created event.
         :rtype: :py:class:`timed_events.TimedEvent`
         """
@@ -176,7 +181,7 @@ class PluginAPI(object):
 
         :param int delay: The number of seconds to schedule the execution
         :param function callback: The handler that will be executed
-        :param \*args: Optional arguments passed to the handler.
+        :param args: Optional arguments passed to the handler.
         :return: The created event.
         :rtype: :py:class:`timed_events.DelayedEvent`
         """
@@ -197,7 +202,7 @@ class PluginAPI(object):
 
         :returns: The current tab.
         """
-        return self.core.current_tab()
+        return self.core.tabs.current_tab
 
     def get_status(self, _):
         """
@@ -214,7 +219,7 @@ class PluginAPI(object):
 
         :param str line: The command to run.
         """
-        return self.core.current_tab().execute_command(*args, **kwargs)
+        return self.core.tabs.current_tab.execute_command(*args, **kwargs)
 
     def all_tabs(self, _):
         """
@@ -329,7 +334,7 @@ class PluginAPI(object):
         :param str event_name: The event name.
         :param function handler: The handler function.
         :param int position: The position of that handler in the handler list.
-            This is useful for plugins like GPG or OTR, which must be the last
+            This is useful for plugins like OTR, which must be the last
             function called on the text.
             Defaults to 0.
 
@@ -368,6 +373,7 @@ class PluginAPI(object):
         """
         self.core.xmpp.del_event_handler(event_name, handler)
 
+
 class BasePlugin(object, metaclass=SafetyMetaclass):
     """
     Class that all plugins derive from.
@@ -379,10 +385,10 @@ class BasePlugin(object, metaclass=SafetyMetaclass):
         self.core = core
         # More hack; luckily we'll never have more than one core object
         SafetyMetaclass.core = core
-        conf = os.path.join(plugins_conf_dir, self.__module__+'.cfg')
+        conf = plugins_conf_dir / (self.__module__ + '.cfg')
         try:
-            self.config = PluginConfig(conf, self.__module__,
-                                       default=self.default_config)
+            self.config = PluginConfig(
+                conf, self.__module__, default=self.default_config)
         except Exception:
             log.debug('Error while creating the plugin config', exc_info=True)
             self.config = PluginConfig(conf, self.__module__)
@@ -419,13 +425,24 @@ class BasePlugin(object, metaclass=SafetyMetaclass):
     def unload(self):
         self.cleanup()
 
-    def add_command(self, name, handler, help, completion=None, short='', usage=''):
+    def add_command(self,
+                    name,
+                    handler,
+                    help,
+                    completion=None,
+                    short='',
+                    usage=''):
         """
         Add a global command.
         You cannot overwrite the existing commands.
         """
-        return self.api.add_command(name, handler, help,
-                completion=completion, short=short, usage=usage)
+        return self.api.add_command(
+            name,
+            handler,
+            help,
+            completion=completion,
+            short=short,
+            usage=usage)
 
     def del_command(self, name):
         """
@@ -458,12 +475,25 @@ class BasePlugin(object, metaclass=SafetyMetaclass):
         """
         return self.api.del_tab_key(tab_type, key)
 
-    def add_tab_command(self, tab_type, name, handler, help, completion=None, short='', usage=''):
+    def add_tab_command(self,
+                        tab_type,
+                        name,
+                        handler,
+                        help,
+                        completion=None,
+                        short='',
+                        usage=''):
         """
         Add a command only for a type of tab.
         """
-        return self.api.add_tab_command(tab_type, name, handler, help,
-                completion=completion, short=short, usage=usage)
+        return self.api.add_tab_command(
+            tab_type,
+            name,
+            handler,
+            help,
+            completion=completion,
+            short=short,
+            usage=usage)
 
     def del_tab_command(self, tab_type, name):
         """
