@@ -4,7 +4,6 @@
 #
 # Poezio is free software: you can redistribute it and/or modify
 # it under the terms of the zlib license. See the COPYING file.
-
 """
 Various methods to convert
 shell colors to poezio colors,
@@ -12,19 +11,23 @@ xhtml code to shell colors,
 poezio colors to xhtml code
 """
 
-import base64
-import curses
 import hashlib
 import re
+from base64 import b64encode, b64decode
 from os import path
-from slixmpp.xmlstream import ET
 from urllib.parse import unquote
+from pathlib import Path
 
 from io import BytesIO
 from xml import sax
 from xml.sax import saxutils
+from typing import List, Dict, Optional, Union, Tuple
 
-digits = '0123456789' # never trust the modules
+from slixmpp.xmlstream import ET
+from poezio.config import config
+from poezio.colors import ncurses_color_to_rgb
+
+digits = '0123456789'  # never trust the modules
 
 XHTML_NS = 'http://www.w3.org/1999/xhtml'
 
@@ -177,7 +180,7 @@ colors = {
     'whitesmoke': 255,
     'yellow': 226,
     'yellowgreen': 149
-}
+}  # type: Dict[str, int]
 
 whitespace_re = re.compile(r'\s+')
 
@@ -188,8 +191,11 @@ poezio_format_trim = re.compile(r'(\x19\d+}|\x19\d|\x19[buaio]|\x19o)+\x19o')
 
 xhtml_simple_attr_re = re.compile(r'\x19\d')
 
-def get_body_from_message_stanza(message, use_xhtml=False,
-                                 tmp_dir=None, extract_images=False):
+
+def get_body_from_message_stanza(message,
+                                 use_xhtml=False,
+                                 extract_images_to: Optional[Path] = None
+                                 ) -> str:
     """
     Returns a string with xhtml markups converted to
     poezio colors if there's an xhtml_im element, or
@@ -198,68 +204,59 @@ def get_body_from_message_stanza(message, use_xhtml=False,
     if not use_xhtml:
         return message['body']
     xhtml = message.xml.find('{http://jabber.org/protocol/xhtml-im}html')
-    if not xhtml:
+    if xhtml is None:
         return message['body']
     xhtml_body = xhtml.find('{http://www.w3.org/1999/xhtml}body')
-    if not xhtml_body:
+    if xhtml_body is None:
         return message['body']
-    content = xhtml_to_poezio_colors(xhtml_body, tmp_dir=tmp_dir,
-                                     extract_images=extract_images)
+    content = xhtml_to_poezio_colors(xhtml_body, tmp_dir=extract_images_to)
     content = content if content else message['body']
     return content or " "
 
-def ncurses_color_to_html(color):
+
+def rgb_to_html(rgb: Tuple[float, float, float]) -> str:
+    """Get the RGB HTML value"""
+    r, g, b = rgb
+    return '#%02X%02X%02X' % (round(r * 255), round(g * 255), round(b * 255))
+
+
+def ncurses_color_to_html(color: int) -> str:
     """
     Takes an int between 0 and 256 and returns
     a string of the form #XXXXXX representing an
     html color.
     """
-    if color <= 15:
-        try:
-            (r, g, b) = curses.color_content(color)
-        except: # fallback in faulty terminals (e.g. xterm)
-            (r, g, b) = curses.color_content(color%8)
-        r = r / 1000 * 6 - 0.01
-        g = g / 1000 * 6 - 0.01
-        b = b / 1000 * 6 - 0.01
-    elif color <= 231:
-        color = color - 16
-        r = color % 6
-        color = color / 6
-        g = color % 6
-        color = color / 6
-        b = color % 6
-    else:
-        color -= 232
-        r = g = b = color / 24 * 6
-    return '#%02X%02X%02X' % (int(r*256/6), int(g*256/6), int(b*256/6))
+    return rgb_to_html(ncurses_color_to_rgb(color))
 
-def parse_css(css):
-    def get_color(value):
-        if value[0] == '#':
-            value = value[1:]
-            length = len(value)
-            if length != 3 and length != 6:
-                return -1
-            value = int(value, 16)
-            if length == 6:
-                r = int(value >> 16)
-                g = int((value >> 8) & 0xff)
-                b = int(value & 0xff)
-                if r == g == b:
-                    return 232 + int(r/10.6251)
-                div = 42.51
-            else:
-                r = int(value >> 8)
-                g = int((value >> 4) & 0xf)
-                b = int(value & 0xf)
-                if r == g == b:
-                    return 232 + int(1.54*r)
-                div = 2.51
-            return 6*6*int(r/div) + 6*int(g/div) + int(b/div) + 16
-        if value in colors:
-            return colors[value]
-        return -1
+
+def _parse_css_color(name: str) -> int:
+    if name[0] == '#':
+        name = name[1:]
+        length = len(name)
+        if length != 3 and length != 6:
+            return -1
+        value = int(name, 16)
+        if length == 6:
+            r = value >> 16
+            g = (value >> 8) & 0xff
+            b = value & 0xff
+            if r == g == b:
+                return int(232 + 0.0941 * r)
+            mult = 0.0235
+        else:
+            r = value >> 8
+            g = (value >> 4) & 0xf
+            b = value & 0xf
+            if r == g == b:
+                return int(232 + 1.54 * r)
+            mult = 0.3984
+        return 6 * 6 * int(mult * r) + 6 * int(mult * g) + int(mult * b) + 16
+    if name in colors:
+        return colors[name]
+    return -1
+
+
+def _parse_css(css: str) -> str:
     shell = ''
     rules = css.split(';')
     for rule in rules:
@@ -269,9 +266,9 @@ def parse_css(css):
         key = key.strip()
         value = value.strip()
         if key == 'background-color':
-            pass#shell += '\x191'
+            pass  #shell += '\x191'
         elif key == 'color':
-            color = get_color(value)
+            color = _parse_css_color(value)
             if color != -1:
                 shell += '\x19%d}' % color
         elif key == 'font-style':
@@ -289,29 +286,40 @@ def parse_css(css):
                 shell += '\x19a'
     return shell
 
-def trim(string):
+
+def _trim(string: str) -> str:
     return re.sub(whitespace_re, ' ', string)
 
+
+def get_hash(data: bytes) -> str:
+    # Currently using SHA-256, this might change in the future.
+    # base64 gives shorter hashes than hex, so use that.
+    return b64encode(hashlib.sha256(data).digest()).rstrip(b'=').replace(
+        b'/', b'-').decode()
+
+
 class XHTMLHandler(sax.ContentHandler):
-    def __init__(self, force_ns=False, tmp_dir=None, extract_images=False):
-        self.builder = []
-        self.formatting = []
-        self.attrs = []
-        self.list_state = []
+    def __init__(self, force_ns=False,
+                 tmp_image_dir: Optional[Path] = None) -> None:
+        self.builder = []  # type: List[str]
+        self.formatting = []  # type: List[str]
+        self.attrs = []  # type: List[Dict[str, str]]
+        self.list_state = []  #  type: List[Union[str, int]]
         self.is_pre = False
         self.a_start = 0
         # do not care about xhtml-in namespace
         self.force_ns = force_ns
 
-        self.tmp_dir = tmp_dir
-        self.extract_images = extract_images
+        self.tmp_image_dir = Path(tmp_image_dir) if tmp_image_dir else None
+        self.enable_css_parsing = config.get('enable_css_parsing')
 
     @property
-    def result(self):
-        sanitized = re.sub(poezio_color_double, r'\1', ''.join(self.builder).strip())
+    def result(self) -> str:
+        sanitized = re.sub(poezio_color_double, r'\1',
+                           ''.join(self.builder).strip())
         return re.sub(poezio_format_trim, '\x19o', sanitized)
 
-    def append_formatting(self, formatting):
+    def append_formatting(self, formatting: str):
         self.formatting.append(formatting)
         self.builder.append(formatting)
 
@@ -319,19 +327,22 @@ class XHTMLHandler(sax.ContentHandler):
         self.formatting.pop()
         self.builder.append('\x19o' + ''.join(self.formatting))
 
-    def characters(self, characters):
-        self.builder.append(characters if self.is_pre else trim(characters))
+    def characters(self, characters: str):
+        self.builder.append(characters if self.is_pre else _trim(characters))
 
     def startElementNS(self, name, _, attrs):
         if name[0] != XHTML_NS and not self.force_ns:
             return
 
         builder = self.builder
-        attrs = {name: value for ((ns, name), value) in attrs.items() if ns is None}
+        attrs = {
+            name: value
+            for ((ns, name), value) in attrs.items() if ns is None
+        }
         self.attrs.append(attrs)
 
-        if 'style' in attrs:
-            style = parse_css(attrs['style'])
+        if 'style' in attrs and self.enable_css_parsing:
+            style = _parse_css(attrs['style'])
             self.append_formatting(style)
 
         name = name[1]
@@ -347,13 +358,17 @@ class XHTMLHandler(sax.ContentHandler):
         elif name == 'em':
             self.append_formatting('\x19i')
         elif name == 'img':
-            if re.match(xhtml_data_re, attrs['src']) and self.extract_images:
-                type_, data = [i for i in re.split(xhtml_data_re, attrs['src']) if i]
-                bin_data = base64.b64decode(unquote(data))
-                filename = hashlib.sha1(bin_data).hexdigest() + '.' + type_
-                filepath = path.join(self.tmp_dir, filename)
+            if re.match(xhtml_data_re,
+                        attrs['src']) and self.tmp_image_dir is not None:
+                type_, data = [
+                    i for i in re.split(xhtml_data_re, attrs['src']) if i
+                ]
+                bin_data = b64decode(unquote(data))
+                filename = get_hash(bin_data) + '.' + type_
+                filepath = self.tmp_image_dir / filename
                 if not path.exists(filepath):
                     try:
+                        self.tmp_image_dir.mkdir(parents=True, exist_ok=True)
                         with open(filepath, 'wb') as fd:
                             fd.write(bin_data)
                         builder.append('[file stored as %s]' % filename)
@@ -362,9 +377,9 @@ class XHTMLHandler(sax.ContentHandler):
                 else:
                     builder.append('[file stored as %s]' % filename)
             else:
-                builder.append(trim(attrs['src']))
+                builder.append(_trim(attrs['src']))
             if 'alt' in attrs:
-                builder.append(' (%s)' % trim(attrs['alt']))
+                builder.append(' (%s)' % _trim(attrs['alt']))
         elif name == 'ul':
             self.list_state.append('ul')
         elif name == 'ol':
@@ -399,11 +414,13 @@ class XHTMLHandler(sax.ContentHandler):
         if name == 'a':
             self.pop_formatting()
             # do not display the link twice
-            text_elements = [x for x in self.builder[self.a_start:]
-                               if not x.startswith('\x19')]
+            text_elements = [
+                x for x in self.builder[self.a_start:]
+                if not x.startswith('\x19')
+            ]
             link_text = ''.join(text_elements).strip()
             if 'href' in attrs and attrs['href'] != link_text:
-                builder.append(' (%s)' % trim(attrs['href']))
+                builder.append(' (%s)' % _trim(attrs['href']))
         elif name == 'blockquote':
             builder.append('”')
         elif name in ('cite', 'em', 'strong'):
@@ -414,27 +431,29 @@ class XHTMLHandler(sax.ContentHandler):
             builder.append('\n')
             self.is_pre = False
 
-        if 'style' in attrs:
+        if 'style' in attrs and self.enable_css_parsing:
             self.pop_formatting()
 
         if 'title' in attrs:
             builder.append(' [' + attrs['title'] + ']')
 
-def xhtml_to_poezio_colors(xml, force=False, tmp_dir=None, extract_images=None):
+
+def xhtml_to_poezio_colors(xml, force=False,
+                           tmp_dir: Optional[Path] = None) -> str:
     if isinstance(xml, str):
         xml = xml.encode('utf8')
     elif not isinstance(xml, bytes):
         xml = ET.tostring(xml)
 
-    handler = XHTMLHandler(force_ns=force, tmp_dir=tmp_dir,
-                           extract_images=extract_images)
+    handler = XHTMLHandler(force_ns=force, tmp_image_dir=tmp_dir)
     parser = sax.make_parser()
     parser.setFeature(sax.handler.feature_namespaces, True)
     parser.setContentHandler(handler)
     parser.parse(BytesIO(xml))
     return handler.result
 
-def clean_text(s):
+
+def clean_text(s: str) -> str:
     """
     Remove all xhtml-im attributes (\x19etc) from the string with the
     complete color format, i.e \x19xxx}
@@ -442,33 +461,49 @@ def clean_text(s):
     s = re.sub(xhtml_attr_re, "", s)
     return s
 
-def clean_text_simple(string):
+
+def clean_text_simple(string: str) -> str:
     """
     Remove all \x19 from the string formatted with simple colors:
     \x198
     """
     pos = string.find('\x19')
     while pos != -1:
-        string = string[:pos] + string[pos+2:]
+        string = string[:pos] + string[pos + 2:]
         pos = string.find('\x19')
     return string
 
-def convert_simple_to_full_colors(text):
+
+def convert_simple_to_full_colors(text: str) -> str:
     """
     takes a \x19n formatted string and returns
     a \x19n} formatted one.
     """
     # TODO, have a single list of this. This is some sort of
-    # dusplicate from windows.format_chars
-    mapping = str.maketrans({'\x0E': '\x19b', '\x0F': '\x19o', '\x10': '\x19u',
-                             '\x11': '\x191', '\x12': '\x192', '\x13': '\x193',
-                             '\x14': '\x194', '\x15': '\x195', '\x16': '\x196',
-                             '\x17': '\x197', '\x18': '\x198', '\x19': '\x199'})
+    # duplicate from windows.format_chars
+    mapping = str.maketrans({
+        '\x0E': '\x19b',
+        '\x0F': '\x19o',
+        '\x10': '\x19u',
+        '\x11': '\x191',
+        '\x12': '\x192',
+        '\x13': '\x193',
+        '\x14': '\x194',
+        '\x15': '\x195',
+        '\x16': '\x196',
+        '\x17': '\x197',
+        '\x18': '\x198',
+        '\x19': '\x199',
+        '\x1A': '\x19i'
+    })
     text = text.translate(mapping)
+
     def add_curly_bracket(match):
         return match.group(0) + '}'
+
     return re.sub(xhtml_simple_attr_re, add_curly_bracket, text)
 
+
 number_to_color_names = {
     1: 'red',
     2: 'green',
@@ -477,12 +512,14 @@ number_to_color_names = {
     5: 'violet',
     6: 'turquoise',
     7: 'white'
-}
+}  # type: Dict[int, str]
 
-def format_inline_css(_dict):
+
+def format_inline_css(_dict: Dict[str, str]) -> str:
     return ''.join(('%s: %s;' % (key, value) for key, value in _dict.items()))
 
-def poezio_colors_to_html(string):
+
+def poezio_colors_to_html(string: str) -> str:
     """
     Convert poezio colors to html
     (e.g. \x191}: <span style='color: red'>)
@@ -490,7 +527,7 @@ def poezio_colors_to_html(string):
     # Maintain a list of the current css attributes used
     # And check if a tag is open (by design, we only open
     # spans tag, and they cannot be nested.
-    current_attrs = {}
+    current_attrs = {}  # type: Dict[str, str]
     tag_open = False
     next_attr_char = string.find('\x19')
     build = ["<body xmlns='http://www.w3.org/1999/xhtml'><p>"]
@@ -505,11 +542,12 @@ def poezio_colors_to_html(string):
             build.append('</span>')
 
     while next_attr_char != -1:
-        attr_char = string[next_attr_char+1].lower()
+        attr_char = string[next_attr_char + 1].lower()
 
         if next_attr_char != 0 and string[:next_attr_char]:
             if current_attrs and not tag_open:
-                build.append('<span style="%s">' % format_inline_css(current_attrs))
+                build.append(
+                    '<span style="%s">' % format_inline_css(current_attrs))
                 tag_open = True
             build.append(saxutils.escape(string[:next_attr_char]))
 
@@ -522,17 +560,21 @@ def poezio_colors_to_html(string):
             check_property('font-weight', 'bold')
         elif attr_char == 'u':
             check_property('text-decoration', 'underline')
+        elif attr_char == 'i':
+            check_property('font-style', 'italic')
 
         if attr_char in digits:
-            number_str = string[next_attr_char+1:string.find('}', next_attr_char)]
+            number_str = string[next_attr_char +
+                                1:string.find('}', next_attr_char)]
             number = int(number_str)
             if number in number_to_color_names:
-                check_property('color', number_to_color_names.get(number, 'black'))
+                check_property('color',
+                               number_to_color_names.get(number, 'black'))
             else:
                 check_property('color', ncurses_color_to_html(number))
-            string = string[next_attr_char+len(number_str)+2:]
+            string = string[next_attr_char + len(number_str) + 2:]
         else:
-            string = string[next_attr_char+2:]
+            string = string[next_attr_char + 2:]
         next_attr_char = string.find('\x19')
 
     if current_attrs and not tag_open and string:
