@@ -10,8 +10,8 @@ Command
         **Usage:** ``/mirror``
 
 """
-from plugin import BasePlugin
-import tabs
+from poezio.plugin import BasePlugin
+from poezio import tabs
 
 class Plugin(BasePlugin):
     def init(self):
