@@ -16,7 +16,7 @@ Just use the word in a message. It will be replaced automatically.
 
 """
 
-from plugin import BasePlugin
+from poezio.plugin import BasePlugin
 import re
 
 class Plugin(BasePlugin):
