@@ -6,15 +6,15 @@ import logging
 
 log = logging.getLogger(__name__)
 
-import os
 from xml.etree import cElementTree as ET
 
+from slixmpp.exceptions import XMPPError
+from slixmpp.xmlstream.xmlstream import NotConnectedError
 from slixmpp.xmlstream.stanzabase import StanzaBase
 from slixmpp.xmlstream.handler import Callback
 from slixmpp.xmlstream.matcher import StanzaPath
 
 from poezio import common
-from poezio import fixes
 from poezio import pep
 from poezio import tabs
 from poezio.bookmarks import Bookmark
@@ -44,23 +44,19 @@ class CommandCore:
             buff = ['Global commands:']
             for name, command in self.core.commands.items():
                 if isinstance(command, Command):
-                    acc.append('  \x19%s}%s\x19o - %s' % (
-                                   color,
-                                   name,
-                                   command.short_desc))
+                    acc.append('  \x19%s}%s\x19o - %s' % (color, name,
+                                                          command.short_desc))
                 else:
                     acc.append('  \x19%s}%s\x19o' % (color, name))
             acc = sorted(acc)
             buff.extend(acc)
             acc = []
             buff.append('Tab-specific commands:')
-            tab_commands = self.core.current_tab().commands
+            tab_commands = self.core.tabs.current_tab.commands
             for name, command in tab_commands.items():
                 if isinstance(command, Command):
-                    acc.append('  \x19%s}%s\x19o - %s' % (
-                                    color,
-                                    name,
-                                    command.short_desc))
+                    acc.append('  \x19%s}%s\x19o - %s' % (color, name,
+                                                          command.short_desc))
                 else:
                     acc.append('  \x19%s}%s\x19o' % (color, name))
             acc = sorted(acc)
@@ -71,7 +67,7 @@ class CommandCore:
         else:
             command = args[0].lstrip('/').strip()
 
-            tab_commands = self.core.current_tab().commands
+            tab_commands = self.core.tabs.current_tab.commands
             if command in tab_commands:
                 tup = tab_commands[command]
             elif command in self.core.commands:
@@ -91,11 +87,13 @@ class CommandCore:
         """
         /runkey <key>
         """
+
         def replace_line_breaks(key):
             "replace ^J with \n"
             if key == '^J':
                 return '\n'
             return key
+
         if args is None:
             return self.help('runkey')
         char = args[0]
@@ -115,7 +113,7 @@ class CommandCore:
         if args is None:
             return self.help('status')
 
-        if not args[0] in POSSIBLE_SHOW.keys():
+        if args[0] not in POSSIBLE_SHOW.keys():
             return self.help('status')
 
         show = POSSIBLE_SHOW[args[0]]
@@ -127,13 +125,14 @@ class CommandCore:
         pres['type'] = show
         self.core.events.trigger('send_normal_presence', pres)
         pres.send()
-        current = self.core.current_tab()
+        current = self.core.tabs.current_tab
         is_muctab = isinstance(current, tabs.MucTab)
         if is_muctab and current.joined and show in ('away', 'xa'):
             current.send_chat_state('inactive')
         for tab in self.core.tabs:
             if isinstance(tab, tabs.MucTab) and tab.joined:
-                muc.change_show(self.core.xmpp, tab.name, tab.own_nick, show, msg)
+                muc.change_show(self.core.xmpp, tab.name, tab.own_nick, show,
+                                msg)
             if hasattr(tab, 'directed_presence'):
                 del tab.directed_presence
         self.core.set_status(show, msg)
@@ -148,41 +147,43 @@ class CommandCore:
         if args is None:
             return self.help('presence')
 
-        jid, type, status = args[0], args[1], args[2]
-        if jid == '.' and isinstance(self.core.current_tab(), tabs.ChatTab):
-            jid = self.core.current_tab().name
-        if type == 'available':
-            type = None
+        jid, ptype, status = args[0], args[1], args[2]
+        if jid == '.' and isinstance(self.core.tabs.current_tab, tabs.ChatTab):
+            jid = self.core.tabs.current_tab.name
+        if ptype == 'available':
+            ptype = None
         try:
-            pres = self.core.xmpp.make_presence(pto=jid, ptype=type, pstatus=status)
+            pres = self.core.xmpp.make_presence(
+                pto=jid, ptype=ptype, pstatus=status)
             self.core.events.trigger('send_normal_presence', pres)
             pres.send()
-        except:
+        except (XMPPError, NotConnectedError):
             self.core.information('Could not send directed presence', 'Error')
-            log.debug('Could not send directed presence to %s', jid, exc_info=True)
+            log.debug(
+                'Could not send directed presence to %s', jid, exc_info=True)
             return
-        tab = self.core.get_tab_by_name(jid)
+        tab = self.core.tabs.by_name(jid)
         if tab:
-            if type in ('xa', 'away'):
+            if ptype in ('xa', 'away'):
                 tab.directed_presence = False
                 chatstate = 'inactive'
             else:
                 tab.directed_presence = True
                 chatstate = 'active'
-            if tab == self.core.current_tab():
+            if tab == self.core.tabs.current_tab:
                 tab.send_chat_state(chatstate, True)
             if isinstance(tab, tabs.MucTab):
                 for private in tab.privates:
                     private.directed_presence = tab.directed_presence
-                if self.core.current_tab() in tab.privates:
-                    self.core.current_tab().send_chat_state(chatstate, True)
+                if self.core.tabs.current_tab in tab.privates:
+                    self.core.tabs.current_tab.send_chat_state(chatstate, True)
 
     @command_args_parser.quoted(1)
     def theme(self, args=None):
         """/theme <theme name>"""
         if args is None:
             return self.help('theme')
-        self.set('theme %s' % (args[0],))
+        self.set('theme %s' % (args[0], ))
 
     @command_args_parser.quoted(1)
     def win(self, args):
@@ -198,30 +199,21 @@ class CommandCore:
         except ValueError:
             number = -1
             name = name.lower()
-        if number != -1 and self.core.current_tab_nb == number:
+        if number != -1 and self.core.tabs.current_tab == number:
             return
         prev_nb = self.core.previous_tab_nb
-        self.core.previous_tab_nb = self.core.current_tab_nb
-        old_tab = self.core.current_tab()
+        self.core.previous_tab_nb = self.core.tabs.current_tab
+        old_tab = self.core.tabs.current_tab
         if 0 <= number < len(self.core.tabs):
             if not self.core.tabs[number]:
                 self.core.previous_tab_nb = prev_nb
                 return
-            self.core.current_tab_nb = number
+            self.core.tabs.set_current_index(number)
         else:
-            matchs = []
-            for tab in self.core.tabs:
-                for tab_name in tab.matching_names():
-                    if tab_name[1] and name in tab_name[1].lower():
-                        matchs.append((tab_name[0], tab))
-            if not matchs:
-                self.core.previous_tab_nb = prev_nb
+            match = self.core.tabs.find_match(name)
+            if match is None:
                 return
-            tab = min(matchs, key=lambda m: m[0])[1]
-            self.core.current_tab_nb = tab.nb
-        old_tab.on_lose_focus()
-        self.core.current_tab().on_gain_focus()
-        self.core.refresh_window()
+            self.core.tabs.set_current_tab(match)
 
     @command_args_parser.quoted(2)
     def move_tab(self, args):
@@ -231,7 +223,7 @@ class CommandCore:
         if args is None:
             return self.help('move_tab')
 
-        current_tab = self.core.current_tab()
+        current_tab = self.core.tabs.current_tab
         if args[0] == '.':
             args[0] = current_tab.nb
         if args[1] == '.':
@@ -248,10 +240,12 @@ class CommandCore:
                     if not old_tab and value == tab.name:
                         old_tab = tab
                 if not old_tab:
-                    self.core.information("Tab %s does not exist" % args[0], "Error")
+                    self.core.information("Tab %s does not exist" % args[0],
+                                          "Error")
                     return None
                 ref = old_tab.nb
             return ref
+
         old = get_nb_from_value(args[0])
         new = get_nb_from_value(args[1])
         if new is None or old is None:
@@ -259,8 +253,6 @@ class CommandCore:
         result = self.core.insert_tab(old, new)
         if not result:
             self.core.information('Unable to move the tab.', 'Info')
-        else:
-            self.core.current_tab_nb = self.core.tabs.index(current_tab)
         self.core.refresh_window()
 
     @command_args_parser.quoted(0, 1)
@@ -274,45 +266,33 @@ class CommandCore:
         elif args:
             jid = safeJID(args[0])
         else:
-            if not isinstance(self.core.current_tab(), tabs.MucTab):
-                return self.core.information('Please provide a server', 'Error')
-            jid = safeJID(self.core.current_tab().name)
+            if not isinstance(self.core.tabs.current_tab, tabs.MucTab):
+                return self.core.information('Please provide a server',
+                                             'Error')
+            jid = safeJID(self.core.tabs.current_tab.name)
         list_tab = tabs.MucListTab(self.core, jid)
         self.core.add_tab(list_tab, True)
         cb = list_tab.on_muc_list_item_received
-        self.core.xmpp.plugin['xep_0030'].get_items(jid=jid,
-                                                    callback=cb)
+        self.core.xmpp.plugin['xep_0030'].get_items(jid=jid, callback=cb)
 
     @command_args_parser.quoted(1)
     def version(self, args):
         """
         /version <jid>
         """
-        def callback(res):
-            "Callback for /version"
-            if not res:
-                return self.core.information('Could not get the software'
-                                             ' version from %s' % jid,
-                                             'Warning')
-            version = '%s is running %s version %s on %s' % (
-                            jid,
-                            res.get('name') or 'an unknown software',
-                            res.get('version') or 'unknown',
-                            res.get('os') or 'an unknown platform')
-            self.core.information(version, 'Info')
-
         if args is None:
             return self.help('version')
-
         jid = safeJID(args[0])
         if jid.resource or jid not in roster or not roster[jid].resources:
-            fixes.get_version(self.core.xmpp, jid, callback=callback)
+            self.core.xmpp.plugin['xep_0092'].get_version(
+                jid, callback=self.core.handler.on_version_result)
         elif jid in roster:
             for resource in roster[jid].resources:
-                fixes.get_version(self.core.xmpp, resource.jid, callback=callback)
+                self.core.xmpp.plugin['xep_0092'].get_version(
+                    resource.jid, callback=self.core.handler.on_version_result)
 
     def _empty_join(self):
-        tab = self.core.current_tab()
+        tab = self.core.tabs.current_tab
         if not isinstance(tab, (tabs.MucTab, tabs.PrivateTab)):
             return (None, None)
         room = safeJID(tab.name).bare
@@ -336,7 +316,7 @@ class CommandCore:
 
         # happens with /join /nickname, which is OK
         if info.bare == '':
-            tab = self.core.current_tab()
+            tab = self.core.tabs.current_tab
             if not isinstance(tab, tabs.MucTab):
                 room, set_nick = (None, None)
             else:
@@ -349,7 +329,7 @@ class CommandCore:
             # use the server of the current room if available
             # check if the current room's name has a server
             if room.find('@') == -1 and not server_root:
-                tab = self.core.current_tab()
+                tab = self.core.tabs.current_tab
                 if isinstance(tab, tabs.MucTab):
                     if tab.name.find('@') != -1:
                         domain = safeJID(tab.name).domain
@@ -366,7 +346,7 @@ class CommandCore:
         else:
             room, nick = self._parse_join_jid(args[0])
         if not room and not nick:
-            return # nothing was parsed
+            return  # nothing was parsed
 
         room = room.lower()
         if nick == '':
@@ -381,13 +361,13 @@ class CommandCore:
         if room in self.core.pending_invites:
             del self.core.pending_invites[room]
 
-        tab = self.core.get_tab_by_name(room, tabs.MucTab)
+        tab = self.core.tabs.by_name_and_class(room, tabs.MucTab)
         # New tab
         if tab is None:
             tab = self.core.open_new_room(room, nick, password=password)
             tab.join()
         else:
-            self.core.focus_tab_named(tab.name)
+            self.core.focus_tab(tab)
             if tab.own_nick == nick and tab.joined:
                 self.core.information('/join: Nothing to do.', 'Info')
             else:
@@ -396,7 +376,12 @@ class CommandCore:
                 tab.password = password
                 tab.join()
 
-        if tab == self.core.current_tab():
+        if config.get('bookmark_on_join'):
+            method = 'remote' if config.get(
+                'use_remote_bookmarks') else 'local'
+            self._add_bookmark('%s/%s' % (room, nick), True, password, method)
+
+        if tab == self.core.tabs.current_tab:
             tab.refresh()
             self.core.doupdate()
 
@@ -405,7 +390,8 @@ class CommandCore:
         """
         /bookmark_local [room][/nick] [password]
         """
-        if not args and not isinstance(self.core.current_tab(), tabs.MucTab):
+        if not args and not isinstance(self.core.tabs.current_tab,
+                                       tabs.MucTab):
             return
         password = args[1] if len(args) > 1 else None
         jid = args[0] if args else None
@@ -417,7 +403,8 @@ class CommandCore:
         """
         /bookmark [room][/nick] [autojoin] [password]
         """
-        if not args and not isinstance(self.core.current_tab(), tabs.MucTab):
+        if not args and not isinstance(self.core.tabs.current_tab,
+                                       tabs.MucTab):
             return
         jid = args[0] if args else ''
         password = args[2] if len(args) > 2 else None
@@ -435,7 +422,7 @@ class CommandCore:
     def _add_bookmark(self, jid, autojoin, password, method):
         nick = None
         if not jid:
-            tab = self.core.current_tab()
+            tab = self.core.tabs.current_tab
             roomname = tab.name
             if tab.joined and tab.own_nick != self.core.own_nick:
                 nick = tab.own_nick
@@ -447,7 +434,7 @@ class CommandCore:
             info = safeJID(jid)
             roomname, nick = info.bare, info.resource
             if roomname == '':
-                tab = self.core.current_tab()
+                tab = self.core.tabs.current_tab
                 if not isinstance(tab, tabs.MucTab):
                     return
                 roomname = tab.name
@@ -461,11 +448,13 @@ class CommandCore:
             bookmark.nick = nick
         if password:
             bookmark.password = password
+
         def callback(iq):
             if iq["type"] != "error":
                 self.core.information('Bookmark added.', 'Info')
             else:
                 self.core.information("Could not add the bookmarks.", "Info")
+
         self.core.bookmarks.save_local()
         self.core.bookmarks.save_remote(self.core.xmpp, callback)
 
@@ -474,8 +463,7 @@ class CommandCore:
         for tab in self.core.get_tabs(tabs.MucTab):
             bookmark = self.core.bookmarks[tab.name]
             if not bookmark:
-                bookmark = Bookmark(tab.name, autojoin=True,
-                                    method=method)
+                bookmark = Bookmark(tab.name, autojoin=True, method=method)
                 new_bookmarks.append(bookmark)
             else:
                 bookmark.method = method
@@ -483,28 +471,28 @@ class CommandCore:
                 self.core.bookmarks.remove(bookmark)
         new_bookmarks.extend(self.core.bookmarks.bookmarks)
         self.core.bookmarks.set(new_bookmarks)
+
         def _cb(iq):
             if iq["type"] != "error":
                 self.core.information("Bookmarks saved.", "Info")
             else:
-                self.core.information("Could not save the remote bookmarks.", "Info")
+                self.core.information("Could not save the remote bookmarks.",
+                                      "Info")
+
         self.core.bookmarks.save_local()
         self.core.bookmarks.save_remote(self.core.xmpp, _cb)
 
     @command_args_parser.ignored
     def bookmarks(self):
         """/bookmarks"""
-        tab = self.core.get_tab_by_name('Bookmarks', tabs.BookmarksTab)
-        old_tab = self.core.current_tab()
+        tab = self.core.tabs.by_name_and_class('Bookmarks', tabs.BookmarksTab)
+        old_tab = self.core.tabs.current_tab
         if tab:
-            self.core.current_tab_nb = tab.nb
+            self.core.tabs.set_current_tab(tab)
         else:
             tab = tabs.BookmarksTab(self.core, self.core.bookmarks)
             self.core.tabs.append(tab)
-            self.core.current_tab_nb = tab.nb
-        old_tab.on_lose_focus()
-        tab.on_gain_focus()
-        self.core.refresh_window()
+            self.core.tabs.set_current_tab(tab)
 
     @command_args_parser.quoted(0, 1)
     def remove_bookmark(self, args):
@@ -514,10 +502,11 @@ class CommandCore:
             if success:
                 self.core.information('Bookmark deleted', 'Info')
             else:
-                self.core.information('Error while deleting the bookmark', 'Error')
+                self.core.information('Error while deleting the bookmark',
+                                      'Error')
 
         if not args:
-            tab = self.core.current_tab()
+            tab = self.core.tabs.current_tab
             if isinstance(tab, tabs.MucTab) and self.core.bookmarks[tab.name]:
                 self.core.bookmarks.remove(tab.name)
                 self.core.bookmarks.save(self.core.xmpp, callback=cb)
@@ -540,15 +529,17 @@ class CommandCore:
             lines = []
             theme = get_theme()
             for section_name, section in config_dict.items():
-                lines.append('\x19%(section_col)s}[%(section)s]\x19o' %
-                        {
-                            'section': section_name,
-                            'section_col': dump_tuple(theme.COLOR_INFORMATION_TEXT),
-                        })
+                lines.append(
+                    '\x19%(section_col)s}[%(section)s]\x19o' % {
+                        'section': section_name,
+                        'section_col': dump_tuple(
+                            theme.COLOR_INFORMATION_TEXT),
+                    })
                 for option_name, option_value in section.items():
-                    lines.append('%s\x19%s}=\x19o%s' % (option_name,
-                                                        dump_tuple(theme.COLOR_REVISIONS_MESSAGE),
-                                                        option_value))
+                    lines.append(
+                        '%s\x19%s}=\x19o%s' %
+                        (option_name, dump_tuple(
+                            theme.COLOR_REVISIONS_MESSAGE), option_value))
             info = ('Current  options:\n%s' % '\n'.join(lines), 'Info')
         elif len(args) == 1:
             option = args[0]
@@ -562,12 +553,13 @@ class CommandCore:
                 if not section:
                     section = plugin_name
                 option = args[1]
-                if not plugin_name in self.core.plugin_manager.plugins:
-                    file_name = self.core.plugin_manager.plugins_conf_dir
-                    file_name = os.path.join(file_name, plugin_name + '.cfg')
+                if plugin_name not in self.core.plugin_manager.plugins:
+                    file_name = self.core.plugin_manager.plugins_conf_dir / (
+                        plugin_name + '.cfg')
                     plugin_config = PluginConfig(file_name, plugin_name)
                 else:
-                    plugin_config = self.core.plugin_manager.plugins[plugin_name].config
+                    plugin_config = self.core.plugin_manager.plugins[
+                        plugin_name].config
                 value = plugin_config.get(option, default='', section=section)
                 info = ('%s=%s' % (option, value), 'Info')
             else:
@@ -589,19 +581,20 @@ class CommandCore:
                     section = plugin_name
                 option = args[1]
                 value = args[2]
-                if not plugin_name in self.core.plugin_manager.plugins:
-                    file_name = self.core.plugin_manager.plugins_conf_dir
-                    file_name = os.path.join(file_name, plugin_name + '.cfg')
+                if plugin_name not in self.core.plugin_manager.plugins:
+                    file_name = self.core.plugin_manager.plugins_conf_dir / (
+                        plugin_name + '.cfg')
                     plugin_config = PluginConfig(file_name, plugin_name)
                 else:
-                    plugin_config = self.core.plugin_manager.plugins[plugin_name].config
+                    plugin_config = self.core.plugin_manager.plugins[
+                        plugin_name].config
                 info = plugin_config.set_and_save(option, value, section)
             else:
                 if args[0] == '.':
-                    name = safeJID(self.core.current_tab().name).bare
+                    name = safeJID(self.core.tabs.current_tab.name).bare
                     if not name:
-                        self.core.information('Invalid tab to use the "." argument.',
-                                              'Error')
+                        self.core.information(
+                            'Invalid tab to use the "." argument.', 'Error')
                         return
                     section = name
                 else:
@@ -652,7 +645,7 @@ class CommandCore:
         Do a /cycle on each room of the given server.
         If none, do it on the current tab
         """
-        tab = self.core.current_tab()
+        tab = self.core.tabs.current_tab
         message = ""
         if args:
             domain = args[0]
@@ -665,51 +658,44 @@ class CommandCore:
                 return self.core.information("No server specified", "Error")
         for tab in self.core.get_tabs(tabs.MucTab):
             if tab.name.endswith(domain):
-                if tab.joined:
-                    muc.leave_groupchat(tab.core.xmpp,
-                                        tab.name,
-                                        tab.own_nick,
-                                        message)
-                tab.joined = False
-                if tab.name == domain:
-                    self.join('"@%s/%s"' %(tab.name, tab.own_nick))
-                else:
-                    self.join('"%s/%s"' %(tab.name, tab.own_nick))
+                tab.leave_room(message)
+                tab.join()
 
     @command_args_parser.quoted(1)
     def last_activity(self, args):
         """
         /last_activity <jid>
         """
+
         def callback(iq):
             "Callback for the last activity"
             if iq['type'] != 'result':
                 if iq['error']['type'] == 'auth':
-                    self.core.information('You are not allowed to see the '
-                                          'activity of this contact.',
-                                          'Error')
+                    self.core.information(
+                        'You are not allowed to see the '
+                        'activity of this contact.', 'Error')
                 else:
-                    self.core.information('Error retrieving the activity', 'Error')
+                    self.core.information('Error retrieving the activity',
+                                          'Error')
                 return
             seconds = iq['last_activity']['seconds']
             status = iq['last_activity']['status']
             from_ = iq['from']
             if not safeJID(from_).user:
                 msg = 'The uptime of %s is %s.' % (
-                        from_,
-                        common.parse_secs_to_str(seconds))
+                    from_, common.parse_secs_to_str(seconds))
             else:
                 msg = 'The last activity of %s was %s ago%s' % (
-                    from_,
-                    common.parse_secs_to_str(seconds),
-                    (' and his/her last status was %s' % status) if status else '')
+                    from_, common.parse_secs_to_str(seconds),
+                    (' and his/her last status was %s' % status)
+                    if status else '')
             self.core.information(msg, 'Info')
 
         if args is None:
             return self.help('last_activity')
         jid = safeJID(args[0])
-        self.core.xmpp.plugin['xep_0012'].get_last_activity(jid,
-                                                       callback=callback)
+        self.core.xmpp.plugin['xep_0012'].get_last_activity(
+            jid, callback=callback)
 
     @command_args_parser.quoted(0, 2)
     def mood(self, args):
@@ -721,15 +707,14 @@ class CommandCore:
 
         mood = args[0]
         if mood not in pep.MOODS:
-            return self.core.information('%s is not a correct value for a mood.'
-                                             % mood,
-                                          'Error')
+            return self.core.information(
+                '%s is not a correct value for a mood.' % mood, 'Error')
         if len(args) == 2:
             text = args[1]
         else:
             text = None
-        self.core.xmpp.plugin['xep_0107'].publish_mood(mood, text,
-                                                       callback=dumb_callback)
+        self.core.xmpp.plugin['xep_0107'].publish_mood(
+            mood, text, callback=dumb_callback)
 
     @command_args_parser.quoted(0, 3)
     def activity(self, args):
@@ -742,9 +727,8 @@ class CommandCore:
 
         general = args[0]
         if general not in pep.ACTIVITIES:
-            return self.core.information('%s is not a correct value for an activity'
-                                             % general,
-                                         'Error')
+            return self.core.information(
+                '%s is not a correct value for an activity' % general, 'Error')
         specific = None
         text = None
         if length == 2:
@@ -756,11 +740,11 @@ class CommandCore:
             specific = args[1]
             text = args[2]
         if specific and specific not in pep.ACTIVITIES[general]:
-            return self.core.information('%s is not a correct value '
-                                         'for an activity' % specific,
-                                         'Error')
-        self.core.xmpp.plugin['xep_0108'].publish_activity(general, specific, text,
-                                                           callback=dumb_callback)
+            return self.core.information(
+                '%s is not a correct value '
+                'for an activity' % specific, 'Error')
+        self.core.xmpp.plugin['xep_0108'].publish_activity(
+            general, specific, text, callback=dumb_callback)
 
     @command_args_parser.quoted(0, 2)
     def gaming(self, args):
@@ -775,9 +759,8 @@ class CommandCore:
             address = args[1]
         else:
             address = None
-        return self.core.xmpp.plugin['xep_0196'].publish_gaming(name=name,
-                                                                server_address=address,
-                                                                callback=dumb_callback)
+        return self.core.xmpp.plugin['xep_0196'].publish_gaming(
+            name=name, server_address=address, callback=dumb_callback)
 
     @command_args_parser.quoted(2, 1, [None])
     def invite(self, args):
@@ -802,9 +785,9 @@ class CommandCore:
             return
         reason = args[1]
         del self.core.pending_invites[jid.bare]
-        self.core.xmpp.plugin['xep_0045'].decline_invite(jid.bare,
-                                                         self.core.pending_invites[jid.bare],
-                                                         reason)
+        self.core.xmpp.plugin['xep_0045'].decline_invite(
+            jid.bare, self.core.pending_invites[jid.bare], reason)
+
 
 ### Commands without a completion in this class ###
 
@@ -813,8 +796,8 @@ class CommandCore:
         """/invitations"""
         build = ""
         for invite in self.core.pending_invites:
-            build += "%s by %s" % (invite,
-                                   safeJID(self.core.pending_invites[invite]).bare)
+            build += "%s by %s" % (
+                invite, safeJID(self.core.pending_invites[invite]).bare)
         if self.core.pending_invites:
             build = "You are invited to the following rooms:\n" + build
         else:
@@ -840,7 +823,8 @@ class CommandCore:
         self.core.save_config()
         self.core.plugin_manager.disable_plugins()
         self.core.disconnect(msg)
-        self.core.xmpp.add_event_handler("disconnected", self.core.exit, disposable=True)
+        self.core.xmpp.add_event_handler(
+            "disconnected", self.core.exit, disposable=True)
 
     @command_args_parser.quoted(0, 1, [''])
     def destroy_room(self, args):
@@ -850,8 +834,10 @@ class CommandCore:
         room = safeJID(args[0]).bare
         if room:
             muc.destroy_room(self.core.xmpp, room)
-        elif isinstance(self.core.current_tab(), tabs.MucTab) and not args[0]:
-            muc.destroy_room(self.core.xmpp, self.core.current_tab().general_jid)
+        elif isinstance(self.core.tabs.current_tab,
+                        tabs.MucTab) and not args[0]:
+            muc.destroy_room(self.core.xmpp,
+                             self.core.tabs.current_tab.general_jid)
         else:
             self.core.information('Invalid JID: "%s"' % args[0], 'Error')
 
@@ -864,12 +850,15 @@ class CommandCore:
             return self.help('bind')
 
         if not config.silent_set(args[0], args[1], section='bindings'):
-            self.core.information('Unable to write in the config file', 'Error')
+            self.core.information('Unable to write in the config file',
+                                  'Error')
 
         if args[1]:
-            self.core.information('%s is now bound to %s' % (args[0], args[1]), 'Info')
+            self.core.information('%s is now bound to %s' % (args[0], args[1]),
+                                  'Info')
         else:
-            self.core.information('%s is now unbound' % args[0], 'Info')
+            self.core.information(
+                '%s is now reset to the default binding' % args[0], 'Info')
 
     @command_args_parser.raw
     def rawxml(self, args):
@@ -883,7 +872,8 @@ class CommandCore:
         stanza = args
         try:
             stanza = StanzaBase(self.core.xmpp, xml=ET.fromstring(stanza))
-            if stanza.xml.tag == 'iq' and stanza.xml.attrib.get('type') in ('get', 'set'):
+            if stanza.xml.tag == 'iq' and stanza.xml.attrib.get('type') in (
+                    'get', 'set'):
                 iq_id = stanza.xml.attrib.get('id')
                 if not iq_id:
                     iq_id = self.core.xmpp.new_id()
@@ -891,22 +881,19 @@ class CommandCore:
 
                 def iqfunc(iq):
                     "handler for an iq reply"
-                    self.core.information('%s' % iq, 'Iq')
+                    self.core.information(str(iq), 'Iq')
                     self.core.xmpp.remove_handler('Iq %s' % iq_id)
 
                 self.core.xmpp.register_handler(
-                        Callback('Iq %s' % iq_id,
-                            StanzaPath('iq@id=%s' % iq_id),
-                            iqfunc
-                            )
-                        )
+                    Callback('Iq %s' % iq_id, StanzaPath('iq@id=%s' % iq_id),
+                             iqfunc))
             stanza.send()
         except:
             self.core.information('Could not send custom stanza', 'Error')
-            log.debug('/rawxml: Could not send custom stanza (%s)',
-                      repr(stanza),
-                      exc_info=True)
-
+            log.debug(
+                '/rawxml: Could not send custom stanza (%s)',
+                repr(stanza),
+                exc_info=True)
 
     @command_args_parser.quoted(1, 256)
     def load(self, args):
@@ -930,9 +917,9 @@ class CommandCore:
         """
         /plugins
         """
-        self.core.information("Plugins currently in use: %s" %
-                                  repr(list(self.core.plugin_manager.plugins.keys())),
-                              'Info')
+        self.core.information(
+            "Plugins currently in use: %s" % repr(
+                list(self.core.plugin_manager.plugins.keys())), 'Info')
 
     @command_args_parser.quoted(1, 1)
     def message(self, args):
@@ -944,18 +931,23 @@ class CommandCore:
         jid = safeJID(args[0])
         if not jid.user and not jid.domain and not jid.resource:
             return self.core.information('Invalid JID.', 'Error')
-        tab = self.core.get_conversation_by_jid(jid.full, False, fallback_barejid=False)
-        muc = self.core.get_tab_by_name(jid.bare, typ=tabs.MucTab)
+        tab = self.core.get_conversation_by_jid(
+            jid.full, False, fallback_barejid=False)
+        muc = self.core.tabs.by_name_and_class(jid.bare, tabs.MucTab)
         if not tab and not muc:
             tab = self.core.open_conversation_window(jid.full, focus=True)
         elif muc:
-            tab = self.core.get_tab_by_name(jid.full, typ=tabs.PrivateTab)
-            if tab:
-                self.core.focus_tab_named(tab.name)
+            if jid.resource:
+                tab = self.core.tabs.by_name_and_class(jid.full,
+                                                       tabs.PrivateTab)
+                if tab:
+                    self.core.focus_tab(tab)
+                else:
+                    tab = self.core.open_private_window(jid.bare, jid.resource)
             else:
-                tab = self.core.open_private_window(jid.bare, jid.resource)
+                tab = muc
         else:
-            self.core.focus_tab_named(tab.name)
+            self.core.focus_tab(tab)
         if len(args) == 2:
             tab.command_say(args[1])
 
@@ -976,27 +968,23 @@ class CommandCore:
         list_tab = tabs.AdhocCommandsListTab(self.core, jid)
         self.core.add_tab(list_tab, True)
         cb = list_tab.on_list_received
-        self.core.xmpp.plugin['xep_0050'].get_commands(jid=jid, local=False,
-                                                       callback=cb)
+        self.core.xmpp.plugin['xep_0050'].get_commands(
+            jid=jid, local=False, callback=cb)
 
     @command_args_parser.ignored
-    def self(self_):
+    def self_(self):
         """
         /self
         """
-        status = self_.core.get_status()
+        status = self.core.get_status()
         show, message = status.show, status.message
-        nick = self_.core.own_nick
-        jid = self_.core.xmpp.boundjid.full
+        nick = self.core.own_nick
+        jid = self.core.xmpp.boundjid.full
         info = ('Your JID is %s\nYour current status is "%s" (%s)'
-                '\nYour default nickname is %s\nYou are running poezio %s' % (
-                jid,
-                message if message else '',
-                show if show else 'available',
-                nick,
-                config_opts.version))
-        self_.core.information(info, 'Info')
-
+                '\nYour default nickname is %s\nYou are running poezio %s' %
+                (jid, message if message else '', show
+                 if show else 'available', nick, config_opts.version))
+        self.core.information(info, 'Info')
 
     @command_args_parser.ignored
     def reload(self):
@@ -1005,6 +993,6 @@ class CommandCore:
         """
         self.core.reload_config()
 
+
 def dumb_callback(*args, **kwargs):
     "mock callback"
-
