@@ -7,21 +7,24 @@ plugin env.
 
 import os
 from os import path
+from pathlib import Path
 import logging
 
-from poezio import tabs
+from poezio import tabs, xdg
 from poezio.core.structs import Command, Completion
 from poezio.plugin import PluginAPI
 from poezio.config import config
 
 log = logging.getLogger(__name__)
 
-class PluginManager(object):
+
+class PluginManager:
     """
     Plugin Manager
     Contains all the references to the plugins
     And keeps track of everything the plugin has done through the API.
     """
+
     def __init__(self, core):
         self.core = core
         # module name -> module object
@@ -52,10 +55,7 @@ class PluginManager(object):
 
     def disable_plugins(self):
         for plugin in set(self.plugins.keys()):
-            try:
-                self.unload(plugin, notify=False)
-            except:
-                pass
+            self.unload(plugin, notify=False)
 
     def load(self, name, notify=True):
         """
@@ -68,7 +68,8 @@ class PluginManager(object):
             module = None
             loader = self.finder.find_module(name, self.load_path)
             if not loader:
-                self.core.information('Could not find plugin: %s' % name, 'Error')
+                self.core.information('Could not find plugin: %s' % name,
+                                      'Error')
                 return
             module = loader.load_module()
         except Exception as e:
@@ -92,9 +93,8 @@ class PluginManager(object):
         except Exception as e:
             log.error('Error while loading the plugin %s', name, exc_info=True)
             if notify:
-                self.core.information('Unable to load the plugin %s: %s' %
-                                      (name, e),
-                                      'Error')
+                self.core.information(
+                    'Unable to load the plugin %s: %s' % (name, e), 'Error')
             self.unload(name, notify=False)
         else:
             if notify:
@@ -130,17 +130,22 @@ class PluginManager(object):
                     self.core.information('Plugin %s unloaded' % name, 'Info')
             except Exception as e:
                 log.debug("Could not unload plugin %s", name, exc_info=True)
-                self.core.information("Could not unload plugin %s: %s" %
-                                      (name, e),
-                                      'Error')
+                self.core.information(
+                    "Could not unload plugin %s: %s" % (name, e), 'Error')
 
-    def add_command(self, module_name, name, handler, help,
-                    completion=None, short='', usage=''):
+    def add_command(self,
+                    module_name,
+                    name,
+                    handler,
+                    help,
+                    completion=None,
+                    short='',
+                    usage=''):
         """
         Add a global command.
         """
         if name in self.core.commands:
-            raise Exception("Command '%s' already exists" % (name,))
+            raise Exception("Command '%s' already exists" % (name, ))
 
         commands = self.commands[module_name]
         commands[name] = Command(handler, help, completion, short, usage)
@@ -155,8 +160,15 @@ class PluginManager(object):
             if name in self.core.commands:
                 del self.core.commands[name]
 
-    def add_tab_command(self, module_name, tab_type, name, handler, help,
-                        completion=None, short='', usage=''):
+    def add_tab_command(self,
+                        module_name,
+                        tab_type,
+                        name,
+                        handler,
+                        help,
+                        completion=None,
+                        short='',
+                        usage=''):
         """
         Add a command only for a type of Tab.
         """
@@ -164,11 +176,11 @@ class PluginManager(object):
         t = tab_type.__name__
         if name in tab_type.plugin_commands:
             return
-        if not t in commands:
+        if t not in commands:
             commands[t] = []
         commands[t].append((name, handler, help, completion))
-        tab_type.plugin_commands[name] = Command(handler, help,
-                                                      completion, short, usage)
+        tab_type.plugin_commands[name] = Command(handler, help, completion,
+                                                 short, usage)
         for tab in self.core.tabs:
             if isinstance(tab, tab_type):
                 tab.update_commands()
@@ -179,7 +191,7 @@ class PluginManager(object):
         """
         commands = self.tab_commands[module_name]
         t = tab_type.__name__
-        if not t in commands:
+        if t not in commands:
             return
         for command in commands[t]:
             if command[0] == name:
@@ -197,7 +209,7 @@ class PluginManager(object):
         t = tab_type.__name__
         if key in tab_type.plugin_keys:
             return
-        if not t in keys:
+        if t not in keys:
             keys[t] = []
         keys[t].append((key, handler))
         tab_type.plugin_keys[key] = handler
@@ -211,7 +223,7 @@ class PluginManager(object):
         """
         keys = self.tab_keys[module_name]
         t = tab_type.__name__
-        if not t in keys:
+        if t not in keys:
             return
         for _key in keys[t]:
             if _key[0] == key:
@@ -227,7 +239,7 @@ class PluginManager(object):
         already exists.
         """
         if key in self.core.key_func:
-            raise Exception("Key '%s' already exists" % (key,))
+            raise Exception("Key '%s' already exists" % (key, ))
         keys = self.keys[module_name]
         keys[key] = handler
         self.core.key_func[key] = handler
@@ -269,39 +281,45 @@ class PluginManager(object):
         completion function that completes the name of the plugins, from
         all .py files in plugins_dir
         """
-        try:
-            names = set()
-            for path_ in self.load_path:
-                try:
-                    add = set(os.listdir(path_))
-                    names |= add
-                except:
-                    pass
-        except OSError as e:
-            self.core.information('Completion failed: %s' % e, 'Error')
-            return False
-        plugins_files = [name[:-3] for name in names if name.endswith('.py')
-                and name != '__init__.py' and not name.startswith('.')]
+        names = set()
+        for path_ in self.load_path:
+            try:
+                add = set(os.listdir(path_))
+                names |= add
+            except OSError:
+                pass
+        plugins_files = [
+            name[:-3] for name in names if name.endswith('.py')
+            and name != '__init__.py' and not name.startswith('.')
+        ]
         plugins_files.sort()
         position = the_input.get_argument_position(quoted=False)
-        return Completion(the_input.new_completion, plugins_files, position, '',
-                                        quotify=False)
+        return Completion(
+            the_input.new_completion,
+            plugins_files,
+            position,
+            '',
+            quotify=False)
 
     def completion_unload(self, the_input):
         """
         completion function that completes the name of loaded plugins
         """
         position = the_input.get_argument_position(quoted=False)
-        return Completion(the_input.new_completion, sorted(self.plugins.keys()), position,
-                                        '', quotify=False)
+        return Completion(
+            the_input.new_completion,
+            sorted(self.plugins.keys()),
+            position,
+            '',
+            quotify=False)
 
-    def on_plugins_dir_change(self, new_value):
-        self.plugins_dir = new_value
+    def on_plugins_dir_change(self, _, new_value):
+        self.plugins_dir = Path(new_value).expanduser()
         self.check_create_plugins_dir()
         self.fill_load_path()
 
-    def on_plugins_conf_dir_change(self, new_value):
-        self.plugins_conf_dir = new_value
+    def on_plugins_conf_dir_change(self, _, new_value):
+        self.plugins_conf_dir = Path(new_value).expanduser()
         self.check_create_plugins_conf_dir()
 
     def initial_set_plugins_conf_dir(self):
@@ -309,12 +327,8 @@ class PluginManager(object):
         Create the plugins_conf_dir
         """
         plugins_conf_dir = config.get('plugins_conf_dir')
-        if not plugins_conf_dir:
-            config_home = os.environ.get('XDG_CONFIG_HOME')
-            if not config_home:
-                config_home = path.join(os.environ.get('HOME'), '.config')
-            plugins_conf_dir = path.join(config_home, 'poezio', 'plugins')
-        self.plugins_conf_dir = path.expanduser(plugins_conf_dir)
+        self.plugins_conf_dir = Path(plugins_conf_dir).expanduser(
+        ) if plugins_conf_dir else xdg.CONFIG_HOME / 'plugins'
         self.check_create_plugins_conf_dir()
 
     def check_create_plugins_conf_dir(self):
@@ -322,12 +336,14 @@ class PluginManager(object):
         Create the plugins config directory if it does not exist.
         Returns True on success, False on failure.
         """
-        if not os.access(self.plugins_conf_dir, os.R_OK | os.X_OK):
+        if not os.access(str(self.plugins_conf_dir), os.R_OK | os.X_OK):
             try:
-                os.makedirs(self.plugins_conf_dir)
+                self.plugins_conf_dir.mkdir(parents=True, exist_ok=True)
             except OSError:
-                log.error('Unable to create the plugin conf dir: %s',
-                        self.plugins_conf_dir, exc_info=True)
+                log.error(
+                    'Unable to create the plugin conf dir: %s',
+                    self.plugins_conf_dir,
+                    exc_info=True)
                 return False
         return True
 
@@ -336,12 +352,8 @@ class PluginManager(object):
         Set the plugins_dir on start
         """
         plugins_dir = config.get('plugins_dir')
-        plugins_dir = plugins_dir or\
-            path.join(os.environ.get('XDG_DATA_HOME') or\
-                             path.join(os.environ.get('HOME'),
-                                          '.local', 'share'),
-                         'poezio', 'plugins')
-        self.plugins_dir = path.expanduser(plugins_dir)
+        self.plugins_dir = Path(plugins_dir).expanduser(
+        ) if plugins_dir else xdg.DATA_HOME / 'plugins'
         self.check_create_plugins_dir()
 
     def check_create_plugins_dir(self):
@@ -349,12 +361,14 @@ class PluginManager(object):
         Create the plugins directory if it does not exist.
         Returns True on success, False on failure.
         """
-        if not os.access(self.plugins_dir, os.R_OK | os.X_OK):
+        if not os.access(str(self.plugins_dir), os.R_OK | os.X_OK):
             try:
-                os.makedirs(self.plugins_dir, exist_ok=True)
+                self.plugins_dir.mkdir(parents=True, exist_ok=True)
             except OSError:
-                log.error('Unable to create the plugins dir: %s',
-                        self.plugins_dir, exc_info=True)
+                log.error(
+                    'Unable to create the plugins dir: %s',
+                    self.plugins_dir,
+                    exc_info=True)
                 return False
         return True
 
@@ -365,14 +379,14 @@ class PluginManager(object):
 
         self.load_path = []
 
-        default_plugin_path = path.join(path.dirname(path.dirname(__file__)),
-                                        'plugins')
+        default_plugin_path = path.join(
+            path.dirname(path.dirname(__file__)), 'plugins')
 
         if os.access(default_plugin_path, os.R_OK | os.X_OK):
             self.load_path.insert(0, default_plugin_path)
 
-        if os.access(self.plugins_dir, os.R_OK | os.X_OK):
-            self.load_path.append(self.plugins_dir)
+        if os.access(str(self.plugins_dir), os.R_OK | os.X_OK):
+            self.load_path.append(str(self.plugins_dir))
 
         try:
             import poezio_plugins
@@ -381,4 +395,3 @@ class PluginManager(object):
         else:
             if poezio_plugins.__path__:
                 self.load_path.append(list(poezio_plugins.__path__)[0])
-
