@@ -14,12 +14,13 @@ revolving around chats.
 """
 
 import logging
-log = logging.getLogger(__name__)
-
 import string
 import time
 from datetime import datetime
 from xml.etree import cElementTree as ET
+from typing import Any, Callable, Dict, List, Optional
+
+from slixmpp import JID, Message
 
 from poezio.core.structs import Command, Completion, Status
 from poezio import timed_events
@@ -33,64 +34,68 @@ from poezio.text_buffer import TextBuffer
 from poezio.theming import get_theme, dump_tuple
 from poezio.decorators import command_args_parser
 
+log = logging.getLogger(__name__)
+
 # getters for tab colors (lambdas, so that they are dynamic)
 STATE_COLORS = {
-        'disconnected': lambda: get_theme().COLOR_TAB_DISCONNECTED,
-        'scrolled': lambda: get_theme().COLOR_TAB_SCROLLED,
-        'nonempty': lambda: get_theme().COLOR_TAB_NONEMPTY,
-        'joined': lambda: get_theme().COLOR_TAB_JOINED,
-        'message': lambda: get_theme().COLOR_TAB_NEW_MESSAGE,
-        'composing': lambda: get_theme().COLOR_TAB_COMPOSING,
-        'highlight': lambda: get_theme().COLOR_TAB_HIGHLIGHT,
-        'private': lambda: get_theme().COLOR_TAB_PRIVATE,
-        'normal': lambda: get_theme().COLOR_TAB_NORMAL,
-        'current': lambda: get_theme().COLOR_TAB_CURRENT,
-        'attention': lambda: get_theme().COLOR_TAB_ATTENTION,
-    }
+    'disconnected': lambda: get_theme().COLOR_TAB_DISCONNECTED,
+    'scrolled': lambda: get_theme().COLOR_TAB_SCROLLED,
+    'nonempty': lambda: get_theme().COLOR_TAB_NONEMPTY,
+    'joined': lambda: get_theme().COLOR_TAB_JOINED,
+    'message': lambda: get_theme().COLOR_TAB_NEW_MESSAGE,
+    'composing': lambda: get_theme().COLOR_TAB_COMPOSING,
+    'highlight': lambda: get_theme().COLOR_TAB_HIGHLIGHT,
+    'private': lambda: get_theme().COLOR_TAB_PRIVATE,
+    'normal': lambda: get_theme().COLOR_TAB_NORMAL,
+    'current': lambda: get_theme().COLOR_TAB_CURRENT,
+    'attention': lambda: get_theme().COLOR_TAB_ATTENTION,
+}
 VERTICAL_STATE_COLORS = {
-        'disconnected': lambda: get_theme().COLOR_VERTICAL_TAB_DISCONNECTED,
-        'scrolled': lambda: get_theme().COLOR_VERTICAL_TAB_SCROLLED,
-        'nonempty': lambda: get_theme().COLOR_VERTICAL_TAB_NONEMPTY,
-        'joined': lambda: get_theme().COLOR_VERTICAL_TAB_JOINED,
-        'message': lambda: get_theme().COLOR_VERTICAL_TAB_NEW_MESSAGE,
-        'composing': lambda: get_theme().COLOR_VERTICAL_TAB_COMPOSING,
-        'highlight': lambda: get_theme().COLOR_VERTICAL_TAB_HIGHLIGHT,
-        'private': lambda: get_theme().COLOR_VERTICAL_TAB_PRIVATE,
-        'normal': lambda: get_theme().COLOR_VERTICAL_TAB_NORMAL,
-        'current': lambda: get_theme().COLOR_VERTICAL_TAB_CURRENT,
-        'attention': lambda: get_theme().COLOR_VERTICAL_TAB_ATTENTION,
-    }
-
+    'disconnected': lambda: get_theme().COLOR_VERTICAL_TAB_DISCONNECTED,
+    'scrolled': lambda: get_theme().COLOR_VERTICAL_TAB_SCROLLED,
+    'nonempty': lambda: get_theme().COLOR_VERTICAL_TAB_NONEMPTY,
+    'joined': lambda: get_theme().COLOR_VERTICAL_TAB_JOINED,
+    'message': lambda: get_theme().COLOR_VERTICAL_TAB_NEW_MESSAGE,
+    'composing': lambda: get_theme().COLOR_VERTICAL_TAB_COMPOSING,
+    'highlight': lambda: get_theme().COLOR_VERTICAL_TAB_HIGHLIGHT,
+    'private': lambda: get_theme().COLOR_VERTICAL_TAB_PRIVATE,
+    'normal': lambda: get_theme().COLOR_VERTICAL_TAB_NORMAL,
+    'current': lambda: get_theme().COLOR_VERTICAL_TAB_CURRENT,
+    'attention': lambda: get_theme().COLOR_VERTICAL_TAB_ATTENTION,
+}
 
 # priority of the different tab states when using Alt+e
 # higher means more priority, < 0 means not selectable
 STATE_PRIORITY = {
-        'normal': -1,
-        'current': -1,
-        'disconnected': 0,
-        'nonempty': 0.1,
-        'scrolled': 0.5,
-        'joined': 0.8,
-        'composing': 0.9,
-        'message': 1,
-        'highlight': 2,
-        'private': 2,
-        'attention': 3
-    }
+    'normal': -1,
+    'current': -1,
+    'disconnected': 0,
+    'nonempty': 0.1,
+    'scrolled': 0.5,
+    'joined': 0.8,
+    'composing': 0.9,
+    'message': 1,
+    'highlight': 2,
+    'private': 2,
+    'attention': 3
+}
 
 SHOW_NAME = {
-        'dnd': 'busy',
-        'away': 'away',
-        'xa': 'not available',
-        'chat': 'chatty',
-        '': 'available'
-    }
+    'dnd': 'busy',
+    'away': 'away',
+    'xa': 'not available',
+    'chat': 'chatty',
+    '': 'available'
+}
 
-class Tab(object):
+
+class Tab:
     plugin_commands = {}
-    plugin_keys = {}
+    plugin_keys = {}  # type: Dict[str, Callable]
+
     def __init__(self, core):
         self.core = core
+        self.nb = 0
         if not hasattr(self, 'name'):
             self.name = self.__class__.__name__
         self.input = None
@@ -99,24 +104,16 @@ class Tab(object):
         self._prev_state = None
 
         self.need_resize = False
-        self.key_func = {}      # each tab should add their keys in there
-                                # and use them in on_input
-        self.commands = {}      # and their own commands
-
+        self.key_func = {}  # each tab should add their keys in there
+        # and use them in on_input
+        self.commands = {}  # and their own commands
 
     @property
-    def size(self):
+    def size(self) -> int:
         return self.core.size
 
-    @property
-    def nb(self):
-        for index, tab in enumerate(self.core.tabs):
-            if tab == self:
-                return index
-        return len(self.core.tabs)
-
     @staticmethod
-    def tab_win_height():
+    def tab_win_height() -> int:
         """
         Returns 1 or 0, depending on if we are using the vertical tab list
         or not.
@@ -138,25 +135,29 @@ class Tab(object):
         return VERTICAL_STATE_COLORS[self._state]()
 
     @property
-    def state(self):
+    def state(self) -> str:
         return self._state
 
     @state.setter
-    def state(self, value):
-        if not value in STATE_COLORS:
+    def state(self, value: str):
+        if value not in STATE_COLORS:
             log.debug("Invalid value for tab state: %s", value)
         elif STATE_PRIORITY[value] < STATE_PRIORITY[self._state] and \
                 value not in ('current', 'disconnected') and \
                 not (self._state == 'scrolled' and value == 'disconnected'):
-            log.debug("Did not set state because of lower priority, asked: %s, kept: %s", value, self._state)
-        elif self._state == 'disconnected' and value not in ('joined', 'current'):
-            log.debug('Did not set state because disconnected tabs remain visible')
+            log.debug(
+                "Did not set state because of lower priority, asked: %s, kept: %s",
+                value, self._state)
+        elif self._state == 'disconnected' and value not in ('joined',
+                                                             'current'):
+            log.debug(
+                'Did not set state because disconnected tabs remain visible')
         else:
             self._state = value
             if self._state == 'current':
                 self._prev_state = None
 
-    def set_state(self, value):
+    def set_state(self, value: str):
         self._state = value
 
     def save_state(self):
@@ -183,7 +184,33 @@ class Tab(object):
         """
         return False
 
-    def register_command(self, name, func, *, desc='', shortdesc='', completion=None, usage=''):
+    def register_commands_batch(self, commands: List[Dict[str, Any]]):
+        """
+        Add several commands in a row, using a list of dictionaries
+        """
+        for command in commands:
+            name = command['name']
+            func = command['func']
+            desc = command.get('desc', '')
+            shortdesc = command.get('shortdesc', '')
+            completion = command.get('completion')
+            usage = command.get('usage', '')
+            self.register_command(
+                name,
+                func,
+                desc=desc,
+                shortdesc=shortdesc,
+                completion=completion,
+                usage=usage)
+
+    def register_command(self,
+                         name: str,
+                         func: Callable,
+                         *,
+                         desc='',
+                         shortdesc='',
+                         completion: Optional[Callable] = None,
+                         usage=''):
         """
         Add a command
         """
@@ -193,7 +220,7 @@ class Tab(object):
             desc = shortdesc
         self.commands[name] = Command(func, desc, completion, shortdesc, usage)
 
-    def complete_commands(self, the_input):
+    def complete_commands(self, the_input: windows.Input) -> bool:
         """
         Does command completion on the specified input for both global and tab-specific
         commands.
@@ -215,13 +242,6 @@ class Tab(object):
                 # Otherwise we would need to add a useless space before being
                 # able to complete the arguments.
                 hit_copy = set(the_input.hit_list)
-                while not hit_copy:
-                    whitespace = the_input.text.find(' ')
-                    if whitespace == -1:
-                        whitespace = len(the_input.text)
-                    the_input.text = the_input.text[:whitespace-1] + the_input.text[whitespace:]
-                    the_input.new_completion(words, 0)
-                    hit_copy = set(the_input.hit_list)
                 if len(hit_copy) == 1:
                     the_input.do_command(' ')
                     the_input.reset_completion()
@@ -234,18 +254,17 @@ class Tab(object):
                     command = self.commands[command_name]
                 elif command_name in self.core.commands:
                     command = self.core.commands[command_name]
-                else:           # Unknown command, cannot complete
+                else:  # Unknown command, cannot complete
                     return False
                 if command.comp is None:
-                    return False # There's no completion function
-                else:
-                    comp = command.comp(the_input)
-                    if comp:
-                        return comp.run()
-                    return comp
+                    return False  # There's no completion function
+                comp = command.comp(the_input)
+                if comp:
+                    return comp.run()
+                return comp
         return False
 
-    def execute_command(self, provided_text):
+    def execute_command(self, provided_text: str) -> bool:
         """
         Execute the command in the input and return False if
         the input didn't contain a command
@@ -254,11 +273,11 @@ class Tab(object):
         if txt.startswith('/') and not txt.startswith('//') and\
                 not txt.startswith('/me '):
             command = txt.strip().split()[0][1:]
-            arg = txt[2+len(command):] # jump the '/' and the ' '
+            arg = txt[2 + len(command):]  # jump the '/' and the ' '
             func = None
-            if command in self.commands: # check tab-specific commands
+            if command in self.commands:  # check tab-specific commands
                 func = self.commands[command].func
-            elif command in self.core.commands: # check global commands
+            elif command in self.core.commands:  # check global commands
                 func = self.core.commands[command].func
             else:
                 low = command.lower()
@@ -270,10 +289,9 @@ class Tab(object):
                     if self.missing_command_callback is not None:
                         error_handled = self.missing_command_callback(low)
                     if not error_handled:
-                        self.core.information("Unknown command (%s)" %
-                                              (command),
-                                              'Error')
-            if command in ('correct', 'say'): # hack
+                        self.core.information(
+                            "Unknown command (%s)" % (command), 'Error')
+            if command in ('correct', 'say'):  # hack
                 arg = xhtml.convert_simple_to_full_colors(arg)
             else:
                 arg = xhtml.clean_text_simple(arg)
@@ -293,6 +311,12 @@ class Tab(object):
         elif not self.size.core_degrade_y:
             self.core.tab_win.refresh()
 
+    def refresh_input(self):
+        """Refresh the current input if any"""
+        if self.input is not None:
+            self.input.refresh()
+            self.core.doupdate()
+
     def refresh(self):
         """
         Called on each screen refresh (when something has changed)
@@ -305,19 +329,19 @@ class Tab(object):
         """
         return self.name
 
-    def get_nick(self):
+    def get_nick(self) -> str:
         """
         Get the nick of the tab (defaults to its name)
         """
         return self.name
 
-    def get_text_window(self):
+    def get_text_window(self) -> Optional[windows.TextWin]:
         """
         Returns the principal TextWin window, if there's one
         """
         return None
 
-    def on_input(self, key, raw):
+    def on_input(self, key: str, raw: bool):
         """
         raw indicates if the key should activate the associated command or not.
         """
@@ -325,12 +349,12 @@ class Tab(object):
 
     def update_commands(self):
         for c in self.plugin_commands:
-            if not c in self.commands:
+            if c not in self.commands:
                 self.commands[c] = self.plugin_commands[c]
 
     def update_keys(self):
         for k in self.plugin_keys:
-            if not k in self.key_func:
+            if k not in self.key_func:
                 self.key_func[k] = self.plugin_keys[k]
 
     def on_lose_focus(self):
@@ -395,7 +419,7 @@ class Tab(object):
             self.input.on_delete()
         self.closed = True
 
-    def matching_names(self):
+    def matching_names(self) -> List[str]:
         """
         Returns a list of strings that are used to name a tab with the /win
         command.  For example you could switch to a tab that returns
@@ -407,8 +431,8 @@ class Tab(object):
     def __del__(self):
         log.debug('------ Closing tab %s', self.__class__.__name__)
 
-class GapTab(Tab):
 
+class GapTab(Tab):
     def __bool__(self):
         return False
 
@@ -420,7 +444,9 @@ class GapTab(Tab):
         return ''
 
     def refresh(self):
-        log.debug('WARNING: refresh() called on a gap tab, this should not happen')
+        log.debug(
+            'WARNING: refresh() called on a gap tab, this should not happen')
+
 
 class ChatTab(Tab):
     """
@@ -431,33 +457,43 @@ class ChatTab(Tab):
     """
     plugin_commands = {}
     plugin_keys = {}
+    message_type = 'chat'
+
     def __init__(self, core, jid=''):
         Tab.__init__(self, core)
         self.name = jid
         self.text_win = None
+        self.directed_presence = None
         self._text_buffer = TextBuffer()
-        self.chatstate = None   # can be "active", "composing", "paused", "gone", "inactive"
+        self.chatstate = None  # can be "active", "composing", "paused", "gone", "inactive"
         # We keep a reference of the event that will set our chatstate to "paused", so that
         # we can delete it or change it if we need to
         self.timed_event_paused = None
+        self.timed_event_not_paused = None
         # Keeps the last sent message to complete it easily in completion_correct, and to replace it.
-        self.last_sent_message = None
+        self.last_sent_message = {}
         self.key_func['M-v'] = self.move_separator
         self.key_func['M-h'] = self.scroll_separator
         self.key_func['M-/'] = self.last_words_completion
         self.key_func['^M'] = self.on_enter
-        self.register_command('say', self.command_say,
-                usage='<message>',
-                shortdesc='Send the message.')
-        self.register_command('xhtml', self.command_xhtml,
-                usage='<custom xhtml>',
-                shortdesc='Send custom XHTML.')
-        self.register_command('clear', self.command_clear,
-                shortdesc='Clear the current buffer.')
-        self.register_command('correct', self.command_correct,
-                desc='Fix the last message with whatever you want.',
-                shortdesc='Correct the last message.',
-                completion=self.completion_correct)
+        self.register_command(
+            'say',
+            self.command_say,
+            usage='<message>',
+            shortdesc='Send the message.')
+        self.register_command(
+            'xhtml',
+            self.command_xhtml,
+            usage='<custom xhtml>',
+            shortdesc='Send custom XHTML.')
+        self.register_command(
+            'clear', self.command_clear, shortdesc='Clear the current buffer.')
+        self.register_command(
+            'correct',
+            self.command_correct,
+            desc='Fix the last message with whatever you want.',
+            shortdesc='Correct the last message.',
+            completion=self.completion_correct)
         self.chat_state = None
         self.update_commands()
         self.update_keys()
@@ -471,14 +507,18 @@ class ChatTab(Tab):
                 self._text_buffer.add_message(**message)
 
     @property
-    def is_muc(self):
-        return False
+    def general_jid(self) -> JID:
+        return NotImplementedError
 
-    def load_logs(self, log_nb):
+    def load_logs(self, log_nb: int) -> Optional[List[Dict[str, Any]]]:
         logs = logger.get_logs(safeJID(self.name).bare, log_nb)
         return logs
 
-    def log_message(self, txt, nickname, time=None, typ=1):
+    def log_message(self,
+                    txt: str,
+                    nickname: str,
+                    time: Optional[datetime] = None,
+                    typ=1):
         """
         Log the messages in the archives.
         """
@@ -486,22 +526,39 @@ class ChatTab(Tab):
         if not logger.log_message(name, nickname, txt, date=time, typ=typ):
             self.core.information('Unable to write in the log file', 'Error')
 
-    def add_message(self, txt, time=None, nickname=None, forced_user=None,
-                    nick_color=None, identifier=None, jid=None, history=None,
-                    typ=1, highlight=False):
+    def add_message(self,
+                    txt,
+                    time=None,
+                    nickname=None,
+                    forced_user=None,
+                    nick_color=None,
+                    identifier=None,
+                    jid=None,
+                    history=None,
+                    typ=1,
+                    highlight=False):
         self.log_message(txt, nickname, time=time, typ=typ)
-        self._text_buffer.add_message(txt, time=time,
-                nickname=nickname,
-                highlight=highlight,
-                nick_color=nick_color,
-                history=history,
-                user=forced_user,
-                identifier=identifier,
-                jid=jid)
+        self._text_buffer.add_message(
+            txt,
+            time=time,
+            nickname=nickname,
+            highlight=highlight,
+            nick_color=nick_color,
+            history=history,
+            user=forced_user,
+            identifier=identifier,
+            jid=jid)
 
-    def modify_message(self, txt, old_id, new_id, user=None, jid=None, nickname=None):
+    def modify_message(self,
+                       txt,
+                       old_id,
+                       new_id,
+                       user=None,
+                       jid=None,
+                       nickname=None):
         self.log_message(txt, nickname, typ=1)
-        message = self._text_buffer.modify_message(txt, old_id, new_id, time=time, user=user, jid=jid)
+        message = self._text_buffer.modify_message(
+            txt, old_id, new_id, time=time, user=user, jid=jid)
         if message:
             self.text_win.modify_message(old_id, message)
             self.core.refresh_window()
@@ -513,8 +570,8 @@ class ChatTab(Tab):
         Complete the input with words recently said
         """
         # build the list of the recent words
-        char_we_dont_want = string.punctuation+' ’„“”…«»'
-        words = list()
+        char_we_dont_want = string.punctuation + ' ’„“”…«»'
+        words = []
         for msg in self._text_buffer.messages[:-40:-1]:
             if not msg:
                 continue
@@ -545,11 +602,12 @@ class ChatTab(Tab):
         if message:
             message.send()
 
-    def generate_xhtml_message(self, arg):
+    def generate_xhtml_message(self, arg: str) -> Message:
         if not arg:
             return
         try:
-            body = xhtml.clean_text(xhtml.xhtml_to_poezio_colors(arg))
+            body = xhtml.clean_text(
+                xhtml.xhtml_to_poezio_colors(arg, force=True))
             ET.fromstring(arg)
         except:
             self.core.information('Could not send custom xhtml', 'Error')
@@ -562,7 +620,7 @@ class ChatTab(Tab):
         msg['html']['body'] = arg
         return msg
 
-    def get_dest_jid(self):
+    def get_dest_jid(self) -> JID:
         return self.name
 
     @refresh_wrapper.always
@@ -573,15 +631,19 @@ class ChatTab(Tab):
         self._text_buffer.messages = []
         self.text_win.rebuild_everything(self._text_buffer)
 
+    def check_send_chat_state(self):
+        "If we should send a chat state"
+        return True
+
     def send_chat_state(self, state, always_send=False):
         """
         Send an empty chatstate message
         """
-        if not self.is_muc or self.joined:
-            if state in ('active', 'inactive', 'gone') and self.inactive and not always_send:
+        if self.check_send_chat_state():
+            if state in ('active', 'inactive',
+                         'gone') and self.inactive and not always_send:
                 return
-            if (config.get_by_tabname('send_chat_states', self.general_jid)
-                    and self.remote_wants_chatstates is not False):
+            if config.get_by_tabname('send_chat_states', self.general_jid):
                 msg = self.core.xmpp.make_message(self.get_dest_jid())
                 msg['type'] = self.message_type
                 msg['chat_state'] = state
@@ -595,8 +657,7 @@ class ChatTab(Tab):
         on the the current status of the input
         """
         name = self.general_jid
-        if (config.get_by_tabname('send_chat_states', name)
-                and self.remote_wants_chatstates):
+        if config.get_by_tabname('send_chat_states', name):
             needed = 'inactive' if self.inactive else 'active'
             self.cancel_paused_delay()
             if not empty_after:
@@ -616,9 +677,15 @@ class ChatTab(Tab):
         # First, cancel the delay if it already exists, before rescheduling
         # it at a new date
         self.cancel_paused_delay()
-        new_event = timed_events.DelayedEvent(4, self.send_chat_state, 'paused')
+        new_event = timed_events.DelayedEvent(4, self.send_chat_state,
+                                              'paused')
         self.core.add_timed_event(new_event)
         self.timed_event_paused = new_event
+        new_event = timed_events.DelayedEvent(
+            30, self.send_chat_state, 'inactive'
+            if self.inactive else 'active')
+        self.core.add_timed_event(new_event)
+        self.timed_event_not_paused = new_event
 
     def cancel_paused_delay(self):
         """
@@ -629,6 +696,8 @@ class ChatTab(Tab):
         if self.timed_event_paused is not None:
             self.core.remove_timed_event(self.timed_event_paused)
             self.timed_event_paused = None
+            self.core.remove_timed_event(self.timed_event_not_paused)
+            self.timed_event_not_paused = None
 
     @command_args_parser.raw
     def command_correct(self, line):
@@ -645,10 +714,14 @@ class ChatTab(Tab):
 
     def completion_correct(self, the_input):
         if self.last_sent_message and the_input.get_argument_position() == 1:
-            return Completion(the_input.auto_completion, [self.last_sent_message['body']], '', quotify=False)
+            return Completion(
+                the_input.auto_completion, [self.last_sent_message['body']],
+                '',
+                quotify=False)
+        return True
 
     @property
-    def inactive(self):
+    def inactive(self) -> bool:
         """Whether we should send inactive or active as a chatstate"""
         return self.core.status.show in ('xa', 'away') or\
                 (hasattr(self, 'directed_presence') and not self.directed_presence)
@@ -677,23 +750,23 @@ class ChatTab(Tab):
         return self.text_win.scroll_down(1)
 
     def on_scroll_up(self):
-        return self.text_win.scroll_up(self.text_win.height-1)
+        return self.text_win.scroll_up(self.text_win.height - 1)
 
     def on_scroll_down(self):
-        return self.text_win.scroll_down(self.text_win.height-1)
+        return self.text_win.scroll_down(self.text_win.height - 1)
 
     def on_half_scroll_up(self):
-        return self.text_win.scroll_up((self.text_win.height-1) // 2)
+        return self.text_win.scroll_up((self.text_win.height - 1) // 2)
 
     def on_half_scroll_down(self):
-        return self.text_win.scroll_down((self.text_win.height-1) // 2)
+        return self.text_win.scroll_down((self.text_win.height - 1) // 2)
 
     @refresh_wrapper.always
     def scroll_separator(self):
         self.text_win.scroll_to_separator()
 
-class OneToOneTab(ChatTab):
 
+class OneToOneTab(ChatTab):
     def __init__(self, core, jid=''):
         ChatTab.__init__(self, core, jid)
 
@@ -702,21 +775,26 @@ class OneToOneTab(ChatTab):
 
         # Set to true once the first disco is done
         self.__initial_disco = False
-        # change this to True or False when
-        # we know that the remote user wants chatstates, or not.
-        # None means we don’t know yet, and we send only "active" chatstates
-        self._remote_wants_chatstates = None
-        self.remote_supports_attention = True
-        self.remote_supports_receipts = True
         self.check_features()
+        self.register_command(
+            'unquery', self.command_unquery, shortdesc='Close the tab.')
+        self.register_command(
+            'close', self.command_unquery, shortdesc='Close the tab.')
+        self.register_command(
+            'attention',
+            self.command_attention,
+            usage='[message]',
+            shortdesc='Request the attention.',
+            desc='Attention: Request the attention of the contact.  Can also '
+            'send a message along with the attention.')
 
     def remote_user_color(self):
         return dump_tuple(get_theme().COLOR_REMOTE_USER)
 
     def update_status(self, status):
         old_status = self.__status
-        if not (old_status.show != status.show or
-                old_status.message != status.message):
+        if not (old_status.show != status.show
+                or old_status.message != status.message):
             return
         self.__status = status
         hide_status_change = config.get_by_tabname('hide_status_change',
@@ -737,28 +815,6 @@ class OneToOneTab(ChatTab):
             msg += 'show: %s, ' % SHOW_NAME[status.show]
         self.add_message(msg[:-2], typ=2)
 
-    @property
-    def remote_wants_chatstates(self):
-        return self._remote_wants_chatstates
-
-    @remote_wants_chatstates.setter
-    def remote_wants_chatstates(self, value):
-        old_value = self._remote_wants_chatstates
-        self._remote_wants_chatstates = value
-        if (old_value is None and value != None) or \
-                (old_value != value and value != None):
-            ok = get_theme().CHAR_OK
-            nope = get_theme().CHAR_EMPTY
-            support = ok if value else nope
-            if value:
-                msg = '\x19%s}Contact supports chat states [%s].'
-            else:
-                msg = '\x19%s}Contact does not support chat states [%s].'
-            color = dump_tuple(get_theme().COLOR_INFORMATION_TEXT)
-            msg = msg % (color, support)
-            self.add_message(msg, typ=0)
-            self.core.refresh_window()
-
     def ack_message(self, msg_id, msg_jid):
         """
         Ack a message
@@ -784,22 +840,25 @@ class OneToOneTab(ChatTab):
         message = self.generate_xhtml_message(xhtml_data)
         if message:
             message['type'] = 'chat'
-            if self.remote_supports_receipts:
-                message._add_receipt = True
-            if self.remote_wants_chatstates:
-                message['chat_sate'] = 'active'
+            message._add_receipt = True
+            message['chat_sate'] = 'active'
             message.send()
             body = xhtml.xhtml_to_poezio_colors(xhtml_data, force=True)
-            self._text_buffer.add_message(body, nickname=self.core.own_nick,
-                                          identifier=message['id'],)
+            self._text_buffer.add_message(
+                body,
+                nickname=self.core.own_nick,
+                nick_color=get_theme().COLOR_OWN_NICK,
+                identifier=message['id'],
+                jid=self.core.xmpp.boundjid)
             self.refresh()
 
     def check_features(self):
         "check the features supported by the other party"
         if safeJID(self.get_dest_jid()).resource:
             self.core.xmpp.plugin['xep_0030'].get_info(
-                    jid=self.get_dest_jid(), timeout=5,
-                    callback=self.features_checked)
+                jid=self.get_dest_jid(),
+                timeout=5,
+                callback=self.features_checked)
 
     @command_args_parser.raw
     def command_attention(self, message):
@@ -816,6 +875,14 @@ class OneToOneTab(ChatTab):
     def command_say(self, line, correct=False, attention=False):
         pass
 
+    @command_args_parser.ignored
+    def command_unquery(self):
+        """
+        /unquery
+        """
+        self.send_chat_state('gone', always_send=True)
+        self.core.close_tab(self)
+
     def missing_command_callback(self, command_name):
         if command_name not in ('correct', 'attention'):
             return False
@@ -830,69 +897,6 @@ class OneToOneTab(ChatTab):
         self.core.information(msg, 'Info')
         return True
 
-    def _feature_attention(self, features):
-        "Check for the 'attention' features"
-        if 'urn:xmpp:attention:0' in features:
-            self.remote_supports_attention = True
-            self.register_command('attention', self.command_attention,
-                                  usage='[message]',
-                                  shortdesc='Request the attention.',
-                                  desc='Attention: Request the attention of '
-                                       'the contact. Can also send a message'
-                                       ' along with the attention.')
-        else:
-            self.remote_supports_attention = False
-        return self.remote_supports_attention
-
-    def _feature_correct(self, features):
-        "Check for the 'correction' feature"
-        if not 'urn:xmpp:message-correct:0' in features:
-            if 'correct' in self.commands:
-                del self.commands['correct']
-        elif not 'correct' in self.commands:
-            self.register_command('correct', self.command_correct,
-                    desc='Fix the last message with whatever you want.',
-                    shortdesc='Correct the last message.',
-                    completion=self.completion_correct)
-        return 'correct' in self.commands
-
-    def _feature_receipts(self, features):
-        "Check for the 'receipts' feature"
-        if 'urn:xmpp:receipts' in features:
-            self.remote_supports_receipts = True
-        else:
-            self.remote_supports_receipts = False
-        return self.remote_supports_receipts
-
     def features_checked(self, iq):
         "Features check callback"
         features = iq['disco_info'].get_features() or []
-        before = ('correct' in self.commands,
-                  self.remote_supports_attention,
-                  self.remote_supports_receipts)
-        correct = self._feature_correct(features)
-        attention = self._feature_attention(features)
-        receipts = self._feature_receipts(features)
-
-        if (correct, attention, receipts) == before and self.__initial_disco:
-            return
-        else:
-            self.__initial_disco = True
-
-        if not (correct or attention or receipts):
-            return # don’t display anything
-
-        ok = get_theme().CHAR_OK
-        nope = get_theme().CHAR_EMPTY
-
-        correct = ok if correct else nope
-        attention = ok if attention else nope
-        receipts = ok if receipts else nope
-
-        msg = ('\x19%s}Contact supports: correction [%s], '
-               'attention [%s], receipts [%s].')
-        color = dump_tuple(get_theme().COLOR_INFORMATION_TEXT)
-        msg = msg % (color, correct, attention, receipts)
-        self.add_message(msg, typ=0)
-        self.core.refresh_window()
-
